@@ -34,7 +34,7 @@ import requests
 from cwclientlib.cwproxy import date_header_value
 
 
-class AbstractMatcher(object):
+class AbstractMatcher:
     """Abstract class for CWShellCompleter's matchers.
 
     A matcher should implement a ``possible_matches`` method. This
@@ -141,7 +141,7 @@ class DefaultMatcher(AbstractMatcher):
             return self.completer.global_matches(text)
 
 
-class CWShellCompleter(object):
+class CWShellCompleter:
     """Custom auto-completion helper for cubicweb-ctl shell.
 
     ``CWShellCompleter`` provides a ``complete`` method suitable for
