@@ -37,6 +37,13 @@ import ssl
 from typing import List
 
 from .builders import build_trinfo
+from .exceptions import (
+    RemoteValidationError,
+    NoResultError,
+    NoUniqueEntity,
+    TaskFailedError,
+    FailedToGrabCSRFTokenError,
+)
 
 if not getattr(ssl, "HAS_SNI", False):
     try:
@@ -73,7 +80,9 @@ class SignedRequestAuth(requests.auth.AuthBase):
         content_to_sign = (
             req.method
             + req.url
-            + "".join(req.headers.get(field, "") for field in self.get_headers_to_sign())
+            + "".join(
+                req.headers.get(field, "") for field in self.get_headers_to_sign()
+            )
         )
         content_signed = hmac.new(
             self.secret.encode("utf-8"),
@@ -111,23 +120,7 @@ def date_header_value() -> str:
     return datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S GMT")
 
 
-class RemoteValidationError(Exception):
-    pass
-
-
-class NoResultError(Exception):
-    "CWProxy.find*() called but result set is empty"
-
-
-class NoUniqueEntity(Exception):
-    "CWProxy.find_one() called but result set contains more than one entity"
-
-
-class TaskFailedError(Exception):
-    pass
-
-
-class CWProxy(object):
+class CWProxy:
     """CWProxy: A simple helper class to ease building CubicWeb_
         clients. It allows to:
 
@@ -164,6 +157,8 @@ class CWProxy(object):
         self.timeout = timeout
         self._ssl_verify = verify
         self._default_vid = "jsonexport"  # OR 'ejsonexport'?
+        self._csrf_token = None
+        self._requests_session = requests.Session()
         self._base_url = urlparse.urlunparse(self.base_url + ("", None, None, None))
 
     def handle_request(self, method: str, path: str, **kwargs):
@@ -172,16 +167,38 @@ class CWProxy(object):
         Arguments are that of requests.request() except for `path` which will
         be used to build a full URL using the base URL bound to this proxy.
         """
+
         msg = "handle_request() got unexpected keyword argument '{}'"
         for unexpected in ("url", "auth"):
             if unexpected in kwargs:
                 raise TypeError(msg.format(unexpected))
-        url = self.build_url(path)
+
+        default_headers = {
+            "Date": date_header_value(),
+            "Origin": self._base_url,
+        }
+        if method != "GET":
+            if self._csrf_token is None:
+                self._grab_csrf_token()
+            default_headers["X-CSRF-Token"] = self._csrf_token
+
         kwargs["auth"] = self.auth
-        kwargs.setdefault("headers", {}).update({"Date": date_header_value()})
+        kwargs.setdefault("headers", {}).update(default_headers)
         kwargs.setdefault("verify", self._ssl_verify)
         kwargs.setdefault("timeout", self.timeout)
-        return requests.request(method, url, **kwargs)
+
+        url = self.build_url(path)
+        return self._requests_session.request(method, url, **kwargs)
+
+    def _grab_csrf_token(self):
+        response = self._requests_session.get(self._base_url, allow_redirects=False)
+        if "csrf_token" not in response.cookies:
+            raise FailedToGrabCSRFTokenError(
+                f"Couldn't grab a CSRF token on website {self._base_url}, this"
+                " CubicWeb instance is probably using an insecure version of CubicWeb,"
+                " ensure that it is running CubicWeb 3.32.9, 3.33.7 or higher."
+            )
+        self._csrf_token = response.cookies["csrf_token"]
 
     def build_url(self, path: str, query=None):
         """Build the URL to query from self.base_url and the given path
@@ -267,8 +284,6 @@ class CWProxy(object):
                 % (result.status_code, result.content, e)
             )
 
-        return result.json()[0]
-
     def rql(self, rql: str, path="view", **data):
         """Perform an urlencoded POST to /<path> with rql=<rql>
 
@@ -297,7 +312,13 @@ class CWProxy(object):
         headers = {
             "Accept": "application/json",
             "Date": date_header_value(),
+            "Origin": self._base_url,
         }
+
+        if self._csrf_token is None:
+            self._grab_csrf_token()
+        headers["X-CSRF-Token"] = self._csrf_token
+
         params = {
             "url": self.build_url(path),
             "headers": headers,
@@ -305,7 +326,7 @@ class CWProxy(object):
             "auth": self.auth,
             "data": data,
         }
-        return requests.post(**params)
+        return self._requests_session.post(**params)
 
     def rqlio(self, queries):
         """Multiple RQL for reading/writing data from/to a CW instance.
@@ -340,7 +361,7 @@ class CWProxy(object):
             "auth": self.auth,
             "files": files,
         }
-        posted = requests.post(**params)
+        posted = self._requests_session.post(**params)
         if posted.status_code == 500:
             try:
                 cause = posted.json()
@@ -388,7 +409,10 @@ class CWProxy(object):
             for property_name in kwargs
         ]
         if args:
-            rql_request = u"{:s}{:s} {:s}".format(rql_request, sep, ", ".join(args))
+            if rql_request:
+                rql_request = "{:s}{:s} {:s}".format(rql_request, sep, ", ".join(args))
+            else:
+                rql_request = ", ".join(args)
         return rql_request
 
     def _rql_args_query(self, rql_request: str, kwargs, sep=",") -> list:
@@ -628,6 +652,62 @@ class CWProxy(object):
         if not self.exist(entity_type, **kwargs):
             return self.insert(entity_type, **kwargs)
 
+    def _args_to_rql(self, prop_names):
+        return ", ".join(
+            f"X {property_name:s} %({property_name:s})s" for property_name in prop_names
+        )
+
+    def update_by_eid(self, eid: int, **kwargs):
+        """Update entity targeted by eid with given kwargs.
+
+        :param eid: eid of entity to update
+        :param kwargs: dict {attributes: values} to set entity with
+
+        Example::
+
+          >>> self.update_by_eid(11, name='neonime', age=33)
+        """
+        return self.update_batch_by_eid([{"eid": eid, **kwargs}])
+
+    def update_batch_by_eid(self, rows):
+        """Update entities targeted by eid with given kwargs.
+
+        :param rows: list of (eid, {attributes: values}) similar
+        to update_by_eid
+
+        Example::
+
+            >>> self.update_batch_by_eid([{'eid': 1, 'age':33}),
+                                          {'eid': 2, 'name': 'neo'})])
+        """
+        requests = []
+        for row in rows:
+            kwargs = row.copy()
+            del kwargs["eid"]
+            rql_args = self._args_to_rql(kwargs)
+            rql_request = "SET {:s} WHERE X eid %(eid)s".format(rql_args)
+            requests.append((rql_request, row))
+        response = self.rqlio(requests)
+        response.raise_for_status()
+
+    def update_by_restrictions(self, select, **kwargs):
+        """Update all entities that match the select triples (kwargs)
+        with given kwargs.
+
+        :param select: dict {attributes: values} to select entities to update
+        :param kwargs: dict {attributes: values} to set entities with
+
+        Example::
+
+          >>> self.update_by_restrictions({'eid': 11}, name='neonime', age=33)
+        """
+        restrictions = self._set_rql_request("", select)
+        setv = self._set_rql_request("", kwargs)
+        rql_request = "SET {:s} WHERE {:s}" "".format(setv, restrictions)
+        args = {**select, **kwargs}
+        response = self.rqlio([(rql_request, args)])
+        response.raise_for_status()
+
     def delete(self, entity_type: str, **kwargs):
         """Delete all entities that have the given type
         and with given values of attributes.
