@@ -4,7 +4,7 @@ from cubicweb.web.views.basecontrollers import ViewController
 
 
 class TestController(ViewController):
-    __regid__ = 'test-controller'
+    __regid__ = "test-controller"
 
     def publish(self, rset=None):
         return b"coucou"
