@@ -21,6 +21,11 @@ import tempfile
 from unittest import TestCase
 import json
 import yaml
+
+try:
+    from yaml import CDumper as Dumper
+except ImportError:
+    from yaml import Dumper
 from contextlib import contextmanager
 
 from cwclientlib import cwproxy_for
@@ -74,7 +79,11 @@ class CWProxyForTC(TestCase):
             if suffix == ".json":
                 f.write(json.dumps(CFG).encode("utf-8"))
             elif suffix == ".yaml":
-                f.write(yaml.dump(CFG, default_flow_style=False).encode("utf-8"))
+                f.write(
+                    yaml.dump(CFG, Dumper=Dumper, default_flow_style=False).encode(
+                        "utf-8"
+                    )
+                )
             else:
                 f.write(cfg2ini(CFG).encode("utf-8"))
         os.chmod(f.name, mode)
